# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.vcenter.hvc.
#---------------------------------------------------------------------------

"""
The ``com.vmware.vcenter.hvc_client`` module provides classes to manage hybrid
links between a local and remote Platform Service Controller. Usage beyond
VMware Cloud on AWS is not supported.

"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class Links(VapiInterface):
    """
    The ``Links`` class provides methods to create, delete, get information,
    and list hybrid links between the local and foreign Platform Service
    Controller (PSC). Usage beyond VMware Cloud on AWS is not supported.
    """

    _VAPI_SERVICE_ID = 'com.vmware.vcenter.hvc.links'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _LinksStub)
        self._VAPI_OPERATION_IDS = {}

    class Summary(VapiStruct):
        """
        The ``Links.Summary`` class contains information about the hybrid link.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     link=None,
                     display_name=None,
                    ):
            """
            :type  link: :class:`str`
            :param link: Unique identifier for the link.
                When clients pass a value of this class as a parameter, the
                attribute must be an identifier for the resource type:
                ``com.vmware.vcenter.hvc.Links``. When methods return a value of
                this class as a return value, the attribute will be an identifier
                for the resource type: ``com.vmware.vcenter.hvc.Links``.
            :type  display_name: :class:`str`
            :param display_name: The display name is set to the domain name which was set during
                create.
            """
            self.link = link
            self.display_name = display_name
            VapiStruct.__init__(self)


    Summary._set_binding_type(type.StructType(
        'com.vmware.vcenter.hvc.links.summary', {
            'link': type.IdType(resource_types='com.vmware.vcenter.hvc.Links'),
            'display_name': type.StringType(),
        },
        Summary,
        False,
        None))


    class CreateSpec(VapiStruct):
        """
        The ``Links.CreateSpec`` class is the specification used for the hybrid
        link creation.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     psc_hostname=None,
                     port=None,
                     domain_name=None,
                     username=None,
                     password=None,
                     ssl_thumbprint=None,
                     admin_groups=None,
                    ):
            """
            :type  psc_hostname: :class:`str`
            :param psc_hostname: The PSC hostname for the domain to be linked.
            :type  port: :class:`str` or ``None``
            :param port: The HTTPS port of the PSC to be linked.
                If None 443 will be used as default.
            :type  domain_name: :class:`str`
            :param domain_name: The domain to which the PSC belongs.
            :type  username: :class:`str`
            :param username: The administrator username of the PSC.
            :type  password: :class:`str`
            :param password: The administrator password of the PSC.
            :type  ssl_thumbprint: :class:`str` or ``None``
            :param ssl_thumbprint: The ssl thumbprint of the server.
                if None no thumbprint is passed.
            :type  admin_groups: :class:`set` of :class:`str` or ``None``
            :param admin_groups: List of groups to be added to enable administrator access to.
                if None administrator access will not be set.
            """
            self.psc_hostname = psc_hostname
            self.port = port
            self.domain_name = domain_name
            self.username = username
            self.password = password
            self.ssl_thumbprint = ssl_thumbprint
            self.admin_groups = admin_groups
            VapiStruct.__init__(self)


    CreateSpec._set_binding_type(type.StructType(
        'com.vmware.vcenter.hvc.links.create_spec', {
            'psc_hostname': type.StringType(),
            'port': type.OptionalType(type.StringType()),
            'domain_name': type.StringType(),
            'username': type.StringType(),
            'password': type.SecretType(),
            'ssl_thumbprint': type.OptionalType(type.StringType()),
            'admin_groups': type.OptionalType(type.SetType(type.StringType())),
        },
        CreateSpec,
        False,
        None))


    class CertificateInfo(VapiStruct):
        """
        The ``Links.CertificateInfo`` class contains information about the SSL
        certificate for a destination PSC endpoint.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     ssl_thumbprint=None,
                    ):
            """
            :type  ssl_thumbprint: :class:`str`
            :param ssl_thumbprint: The SHA-256 thumbprint of the SSL certificate for the destination
                PSC endpoint.
            """
            self.ssl_thumbprint = ssl_thumbprint
            VapiStruct.__init__(self)


    CertificateInfo._set_binding_type(type.StructType(
        'com.vmware.vcenter.hvc.links.certificate_info', {
            'ssl_thumbprint': type.StringType(),
        },
        CertificateInfo,
        False,
        None))


    class Info(VapiStruct):
        """
        The ``Links.Info`` class contains information about link.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """

        _validator_list = [
            UnionValidator(
                'connection_health_status',
                {
                    'UNHEALTHY' : [('health_status_message', True)],
                    'HEALTHY' : [],
                }
            ),
        ]



        def __init__(self,
                     connection_health_status=None,
                     health_status_message=None,
                    ):
            """
            :type  connection_health_status: :class:`Links.Info.HealthStatus` or ``None``
            :param connection_health_status: Health status of the connection
            :type  health_status_message: :class:`com.vmware.vapi.std_client.LocalizableMessage`
            :param health_status_message: Localizable messages associated with health status
                This attribute is optional and it is only relevant when the value
                of ``connectionHealthStatus`` is
                :attr:`Links.Info.HealthStatus.UNHEALTHY`.
            """
            self.connection_health_status = connection_health_status
            self.health_status_message = health_status_message
            VapiStruct.__init__(self)


        class HealthStatus(Enum):
            """
            The ``Links.Info.HealthStatus`` class defines the possible states for
            health of a link.

            .. note::
                This class represents an enumerated type in the interface language
                definition. The class contains class attributes which represent the
                values in the current version of the enumerated type. Newer versions of
                the enumerated type may contain new values. To use new values of the
                enumerated type in communication with a server that supports the newer
                version of the API, you instantiate this class. See :ref:`enumerated
                type description page <enumeration_description>`.
            """
            HEALTHY = None
            """
            Connection is healthy

            """
            UNHEALTHY = None
            """
            Connection issues will need to be remediated

            """

            def __init__(self, string):
                """
                :type  string: :class:`str`
                :param string: String value for the :class:`HealthStatus` instance.
                """
                Enum.__init__(string)

        HealthStatus._set_values({
            'HEALTHY': HealthStatus('HEALTHY'),
            'UNHEALTHY': HealthStatus('UNHEALTHY'),
        })
        HealthStatus._set_binding_type(type.EnumType(
            'com.vmware.vcenter.hvc.links.info.health_status',
            HealthStatus))

    Info._set_binding_type(type.StructType(
        'com.vmware.vcenter.hvc.links.info', {
            'connection_health_status': type.OptionalType(type.ReferenceType(__name__, 'Links.Info.HealthStatus')),
            'health_status_message': type.OptionalType(type.ReferenceType('com.vmware.vapi.std_client', 'LocalizableMessage')),
        },
        Info,
        False,
        None))


    class Credentials(VapiStruct):
        """
        The ``Links.Credentials`` class specifies user credentials to make a
        successful connection to remote endpoint

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     user_name=None,
                     password=None,
                    ):
            """
            :type  user_name: :class:`str`
            :param user_name: Name of the user to authenticate
            :type  password: :class:`str`
            :param password: Password for the user.
            """
            self.user_name = user_name
            self.password = password
            VapiStruct.__init__(self)


    Credentials._set_binding_type(type.StructType(
        'com.vmware.vcenter.hvc.links.credentials', {
            'user_name': type.StringType(),
            'password': type.SecretType(),
        },
        Credentials,
        False,
        None))



    def create(self,
               spec,
               ):
        """
        Creates a new hybrid link between the local and foreign PSC. Usage
        beyond VMware Cloud on AWS is not supported.

        :type  spec: :class:`Links.CreateSpec`
        :param spec: Specification for the new link to be created.
        :rtype: :class:`str`
        :return: The identifier of the newly linked domain.
            The return value will be an identifier for the resource type:
            ``com.vmware.vcenter.hvc.Links``.
        :raise: :class:`com.vmware.vapi.std.errors_client.AlreadyExists` 
            If the link already exists.
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidArgument` 
            If the values of any of the attributes of the ``spec`` parameter
            are not valid.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unsupported` 
            If the PSC or the VC version is not supported.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            If the user is not authorized.
        :raise: :class:`com.vmware.vapi.std.errors_client.UnverifiedPeer` 
            If the SSL certificate of the foreign PSC cannot be validated by
            comparing with the thumbprint provided in
            :attr:`Links.CreateSpec.ssl_thumbprint` or if
            :attr:`Links.CreateSpec.ssl_thumbprint` is None. The value of the
            {\\\\@link UnverifiedPeer#data) attribute will be a class that
            contains all the attributes defined in
            :class:`Links.CertificateInfo`.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            if the system reports an error while responding to the request.
        """
        return self._invoke('create',
                            {
                            'spec': spec,
                            })

    def delete(self,
               link,
               ):
        """
        Deletes an existing hybrid link. This API should be used when unlink is
        initiated from Cloud Gateway.

        :type  link: :class:`str`
        :param link: Identifier of the hybrid link.
            The parameter must be an identifier for the resource type:
            ``com.vmware.vcenter.hvc.Links``.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            If the hybrid link associated with ``link`` does not exist.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            If the user is not authorized.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            if the system reports an error while responding to the request.
        """
        return self._invoke('delete',
                            {
                            'link': link,
                            })

    def delete_with_credentials(self,
                                link,
                                credentials=None,
                                ):
        """
        Deletes an existing hybrid link. This API should be used when unlink is
        initiated from cloud vCenter. Usage beyond VMware Cloud on AWS is not
        supported.

        :type  link: :class:`str`
        :param link: Identifier of the hybrid link.
            The parameter must be an identifier for the resource type:
            ``com.vmware.vcenter.hvc.Links``.
        :type  credentials: :class:`Links.Credentials` or ``None``
        :param credentials: Credentials to use for authentication to the remote system.
            This parameter is currently required. In the future, it may become
            optional.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            if the hybrid link associated with ``link`` does not exist.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            if the user is not authorized.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            if the system reports an error while responding to the request.
        """
        return self._invoke('delete_with_credentials',
                            {
                            'link': link,
                            'credentials': credentials,
                            })

    def list(self):
        """
        Enumerates the list of registered hybrid links. Usage beyond VMware
        Cloud on AWS is not supported.


        :rtype: :class:`list` of :class:`Links.Summary`
        :return: The :class:`list` of hybrid link information.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            if the system reports an error while responding to the request.
        """
        return self._invoke('list', None)

    def get(self,
            link,
            ):
        """
        Gets information for a link. Usage beyond VMware Cloud on AWS is not
        supported.

        :type  link: :class:`str`
        :param link: Unique identifier of the link.
            The parameter must be an identifier for the resource type:
            ``com.vmware.vcenter.hvc.Links``.
        :rtype: :class:`Links.Info`
        :return: Information about the link.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            if the system reports an error while responding to the request.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            if the hybrid link associated with ``link`` does not exist.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            if the user is not authorized to perform this operation.
        :raise: :class:`com.vmware.vapi.std.errors_client.ResourceBusy` 
            if the service is busy.
        """
        return self._invoke('get',
                            {
                            'link': link,
                            })
class _LinksStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for create operation
        create_input_type = type.StructType('operation-input', {
            'spec': type.ReferenceType(__name__, 'Links.CreateSpec'),
        })
        create_error_dict = {
            'com.vmware.vapi.std.errors.already_exists':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'AlreadyExists'),
            'com.vmware.vapi.std.errors.invalid_argument':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidArgument'),
            'com.vmware.vapi.std.errors.unsupported':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unsupported'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.unverified_peer':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'UnverifiedPeer'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        create_input_value_validator_list = [
        ]
        create_output_validator_list = [
        ]
        create_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/hvc/links',
            path_variables={
            },
             header_parameters={
               },
            query_parameters={
            },
            content_type='application/json'
        )

        # properties for delete operation
        delete_input_type = type.StructType('operation-input', {
            'link': type.IdType(resource_types='com.vmware.vcenter.hvc.Links'),
        })
        delete_error_dict = {
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        delete_input_value_validator_list = [
        ]
        delete_output_validator_list = [
        ]
        delete_rest_metadata = OperationRestMetadata(
            http_method='DELETE',
            url_template='/hvc/links/{link_id}',
            path_variables={
                'link': 'link_id',
            },
             header_parameters={
               },
            query_parameters={
            },
            content_type='application/json'
        )

        # properties for delete_with_credentials operation
        delete_with_credentials_input_type = type.StructType('operation-input', {
            'link': type.IdType(resource_types='com.vmware.vcenter.hvc.Links'),
            'credentials': type.OptionalType(type.ReferenceType(__name__, 'Links.Credentials')),
        })
        delete_with_credentials_error_dict = {
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        delete_with_credentials_input_value_validator_list = [
        ]
        delete_with_credentials_output_validator_list = [
        ]
        delete_with_credentials_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/hvc/links/{link}',
            path_variables={
                'link': 'link',
            },
             header_parameters={
                 },
            query_parameters={
            }
        )

        # properties for list operation
        list_input_type = type.StructType('operation-input', {})
        list_error_dict = {
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        list_input_value_validator_list = [
        ]
        list_output_validator_list = [
        ]
        list_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/hvc/links',
            path_variables={
            },
             header_parameters={
             },
            query_parameters={
            },
            content_type='application/json'
        )

        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'link': type.IdType(resource_types='com.vmware.vcenter.hvc.Links'),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.resource_busy':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ResourceBusy'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/hvc/links/{link}',
            path_variables={
                'link': 'link',
            },
             header_parameters={
               },
            query_parameters={
            }
        )

        operations = {
            'create': {
                'input_type': create_input_type,
                'output_type': type.IdType(resource_types='com.vmware.vcenter.hvc.Links'),
                'errors': create_error_dict,
                'input_value_validator_list': create_input_value_validator_list,
                'output_validator_list': create_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'delete': {
                'input_type': delete_input_type,
                'output_type': type.VoidType(),
                'errors': delete_error_dict,
                'input_value_validator_list': delete_input_value_validator_list,
                'output_validator_list': delete_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'delete_with_credentials': {
                'input_type': delete_with_credentials_input_type,
                'output_type': type.VoidType(),
                'errors': delete_with_credentials_error_dict,
                'input_value_validator_list': delete_with_credentials_input_value_validator_list,
                'output_validator_list': delete_with_credentials_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'list': {
                'input_type': list_input_type,
                'output_type': type.ListType(type.ReferenceType(__name__, 'Links.Summary')),
                'errors': list_error_dict,
                'input_value_validator_list': list_input_value_validator_list,
                'output_validator_list': list_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType(__name__, 'Links.Info'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'create': create_rest_metadata,
            'delete': delete_rest_metadata,
            'delete_with_credentials': delete_with_credentials_rest_metadata,
            'list': list_rest_metadata,
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vcenter.hvc.links',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=True)


class StubFactory(StubFactoryBase):
    _attrs = {
        'Links': Links,
        'management': 'com.vmware.vcenter.hvc.management_client.StubFactory',
    }




"""
SecurityContext API Provider filter
"""

__author__ = 'VMware, Inc.'
__copyright__ = 'Copyright 2017, 2019 VMware, Inc.  All rights reserved. -- VMware Confidential'  # pylint: disable=line-too-long

import abc
import six

from vmware.vapi.core import ExecutionContext
from vmware.vapi.lib.log import get_vapi_logger
from vmware.vapi.provider.filter import ApiProviderFilter


# Configure logging
logger = get_vapi_logger(__name__)


@six.add_metaclass(abc.ABCMeta)
class SecurityContextFilter(ApiProviderFilter):
    """
    SecurityContextFilter in API Provider chain adds the security
    context to the execution context passed in.
    """
    def __init__(self, next_provider=None):
        """
        Initialize SecurityContextFilter

        :type  next_provider: :class:`vmware.vapi.core.ApiProvider`
        :param next_provider: API Provider to invoke the requests
        """
        ApiProviderFilter.__init__(self, next_provider)

    @abc.abstractmethod
    def get_max_retries(self):
        """
        Get the max number of retries

        :rtype: :class:`int`
        :return: Number of retries
        """

    @abc.abstractmethod
    def get_security_context(self, on_error):
        """
        Retrieve security context. If this method is called after an error
        occured and the request needs to be retried, then a new security context
        may be created depending on the scheme.

        :type  on_error: :class:`bool`
        :param on_error: Whether this method is called after getting an error
        :rtype: :class:`vmware.vapi.core.SecurityContext`
        :return: Security context
        """

    @abc.abstractmethod
    def should_retry(self, error_value):
        """
        Returns whether the request should be retried or not based on the error.

        :type  error_value: :class:`vmware.vapi.data.value.ErrorValue`
        :param error_value: Method error
        :rtype: :class:`bool`
        :return: Returns True if request should be retried based on the error
            value provided else False
        """

    def invoke(self, service_id, operation_id, input_value, ctx):   # pylint: disable=R1710
        """
        Invoke an API request

        :type  service_id: :class:`str`
        :param service_id: Service identifier
        :type  operation_id: :class:`str`
        :param operation_id: Operation identifier
        :type  input_value: :class:`vmware.vapi.data.value.StructValue`
        :param input_value: Method input parameters
        :type  ctx: :class:`vmware.vapi.core.ExecutionContext`
        :param ctx: Execution context for this method

        :rtype: :class:`vmware.vapi.core.MethodResult`
        :return: Result of the method invocation
        """
        app_ctx = ctx.application_context if ctx else None
        on_error = False
        for i in range(0, self.get_max_retries() + 1):
            # Typically get_security_context() should always return a valid
            # security context. In case of LegacySecurityContextFilter,
            # get_security_context() may return None if no security context is
            # set and hence needs to be handled.
            sec_ctx = (self.get_security_context(on_error)
                       or (ctx.security_context if ctx else None))
            # Reuse the exist ExecutionContext object if present
            # ExecutionContext object may contain more fields
            if ctx is not None:
                ctx.application_context = app_ctx
                ctx.security_context = sec_ctx
            else:
                ctx = ExecutionContext(app_ctx, sec_ctx)
            method_result = ApiProviderFilter.invoke(
                self, service_id, operation_id, input_value, ctx)
            if (method_result.error is None
                    or not self.should_retry(method_result.error)):
                return method_result
            on_error = True


class LegacySecurityContextFilter(SecurityContextFilter):
    """
    SecurityContextFilter in API Provider chain adds the security
    context to the execution context passed in.
    """
    def __init__(self, next_provider=None, security_context=None):
        """
        Initialize SecurityContextFilter

        :type  next_provider: :class:`vmware.vapi.core.ApiProvider`
        :param next_provider: API Provider to invoke the requests
        :type  security_context: :class:`vmware.vapi.core.SecurityContext`
        :param security_context: Security context
        """
        SecurityContextFilter.__init__(self, next_provider)
        self._security_context = security_context

    def get_max_retries(self):
        """
        Get the max number of retries

        :rtype: :class:`int`
        :return: Number of retries
        """
        return 0

    def set_security_context(self, security_context):
        """
        Set security context

        :type  security_context: :class:`vmware.vapi.core.SecurityContext`
        :param security_context: Security context
        """
        self._security_context = security_context

    def get_security_context(self, on_error):
        """
        Retrieve the stored security context.

        :type  on_error: :class:`bool`
        :param on_error: Whether this method is called after getting an error
        :rtype: :class:`vmware.vapi.core.SecurityContext` or ``None``
        :return: Security context or None
        """
        # This method shouldn't return None but it is violating in cases when no
        # security context is set
        return self._security_context

    def should_retry(self, error_value):
        """
        Returns whether the request should be retried or not based on the error.

        :type  error_value: :class:`vmware.vapi.data.value.ErrorValue`
        :param error_value: Method error
        :rtype: :class:`bool`
        :return: Returns True if request should be retried based on the error
            value provided else False
        """
        return False



"""
Helper classes for creation of resource bundles
"""

__author__ = 'VMware, Inc.'
__copyright__ = 'Copyright 2015 VMware, Inc.  All rights reserved. -- VMware Confidential'  # pylint: disable=line-too-long

import six

from vmware.vapi.message import MessageBundle


def _parse_message_data_sets(data_sets):
    """
    Parse the message data sets and extract the messages

    :type  data_sets: :class:`list` of :class:`str`
    :param data_sets: Resource bundle string extracted from the egg file
    :rtype: :class:`dict` of :class:`str`, :class:`str`
    :return: Dictionary of messages
    """
    result = {}
    for data_set in data_sets:
        messages = data_set.splitlines()
        for message in messages:
            if isinstance(message, six.binary_type):
                message = message.decode('utf-8')
            message = message.strip()
            if not message.startswith('#') and '=' in message:
                key, value = message.split('=', 1)
                key = key.strip()
                # To use '=' in the value, it has to be escaped
                value = value.strip().replace(r'\=', '=')
                if not key:
                    raise ValueError('Key not present for value %s', value)
                if not value:
                    raise ValueError('Value not present for key %s', value)
                result[key] = value
    return result


class PropertiesResourceBundle(MessageBundle):
    """
    Class for creating resource bundles using property files in the
    distributable. i.e. egg or zip file.
    """
    def __init__(self, property_files):
        """
        Initialize PropertiesResourceBundle

        :type  property_files: :class:`list` of :class:`tuple` or :class:`tuple`
        :param property_files: List of property files to be processed. The tuple
            should be of the form (package, resource_name). For ex: If a file
            named runtime.properties is present in vmware.vapi package, the
            tuple to be passed is ('vmware.vapi', 'runtime.properties')
        """
        # Importing here so that there is no dependency on pkg_resources
        # for modules that use other kind of MessageBundle classes
        from pkg_resources import resource_string
        if not isinstance(property_files, list):
            property_files = [property_files]
        bundles = [resource_string(path, filename)
                   for path, filename in property_files]
        messages = _parse_message_data_sets(bundles)
        MessageBundle.__init__(self, messages)


class FileMessageBundle(MessageBundle):
    """
    Class for creating resource bundles using list of files
    that contain messages
    """
    def __init__(self, message_files):
        """
        Initialize FileMessageBundle

        :type  message_files: :class:`list` of :class:`str` or :class:`str`
        :param message_files: List of message files to be processed. Each
            element in the list should be a fully qualified file path.
        """
        if not isinstance(message_files, list):
            message_files = [message_files]
        bundles = []
        for message_file in message_files:
            if six.PY2:
                with open(message_file, 'r') as fp:
                    bundles.append(fp.read())
            else:
                with open(message_file, 'r', encoding='utf-8') as fp:
                    bundles.append(fp.read())
        messages = _parse_message_data_sets(bundles)
        MessageBundle.__init__(self, messages)


class DictionaryResourceBundle(MessageBundle):
    """
    Class for creating resource bundles using dictionary of messages
    """
    def __init__(self, msgs):
        """
        Initialize DictionaryResourceBundle

        :type  msgs: :class:`dict`
        :param msgs: Message bundle
        """
        if not isinstance(msgs, dict):
            raise TypeError('Messages should be a dictionary')
        MessageBundle.__init__(self, msgs)



# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.attestation.
#---------------------------------------------------------------------------

"""
The
``com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.attestation_client``
module provides classes for configuring the Attestation Service. It attests the
state of a remote infrastructure node.

"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from com.vmware.cis_client import Tasks
from vmware.vapi.stdlib.client.task import Task
from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class ServiceStatus(VapiInterface):
    """
    The ``ServiceStatus`` class provides methods to get the Attestation Service
    health status. This class was added in vSphere API 7.0.0.0.
    """

    _VAPI_SERVICE_ID = 'com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.attestation.service_status'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _ServiceStatusStub)
        self._VAPI_OPERATION_IDS = {}
        self._VAPI_OPERATION_IDS.update({'get_task': 'get$task'})

    class Health(Enum):
        """
        The ``ServiceStatus.Health`` class defines the possible service health
        states. This enumeration was added in vSphere API 7.0.0.0.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        NONE = None
        """
        No status available. This class attribute was added in vSphere API 7.0.0.0.

        """
        OK = None
        """
        Service is functioning normally. This class attribute was added in vSphere
        API 7.0.0.0.

        """
        WARNING = None
        """
        Service is functioning, however there is an issue that requires attention.
        This class attribute was added in vSphere API 7.0.0.0.

        """
        ERROR = None
        """
        Service is not functioning. This class attribute was added in vSphere API
        7.0.0.0.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`Health` instance.
            """
            Enum.__init__(string)

    Health._set_values({
        'NONE': Health('NONE'),
        'OK': Health('OK'),
        'WARNING': Health('WARNING'),
        'ERROR': Health('ERROR'),
    })
    Health._set_binding_type(type.EnumType(
        'com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.attestation.service_status.health',
        Health))


    class Info(VapiStruct):
        """
        The ``ServiceStatus.Info`` class contains information that describes the
        status of the service. This class was added in vSphere API 7.0.0.0.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     health=None,
                     details=None,
                    ):
            """
            :type  health: :class:`ServiceStatus.Health`
            :param health: The service health status. This attribute was added in vSphere API
                7.0.0.0.
            :type  details: :class:`list` of :class:`com.vmware.vapi.std_client.LocalizableMessage`
            :param details: Details regarding the health of the service. 
                
                When the service ``ServiceStatus.Health`` is not
                :attr:`ServiceStatus.Health.OK` or
                :attr:`ServiceStatus.Health.NONE`, this member will provide an
                actionable description of the issues present.. This attribute was
                added in vSphere API 7.0.0.0.
            """
            self.health = health
            self.details = details
            VapiStruct.__init__(self)


    Info._set_binding_type(type.StructType(
        'com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.attestation.service_status.info', {
            'health': type.ReferenceType(__name__, 'ServiceStatus.Health'),
            'details': type.ListType(type.ReferenceType('com.vmware.vapi.std_client', 'LocalizableMessage')),
        },
        Info,
        False,
        None))




    def get_task(self,
            cluster,
            ):
        """
        Return the Attestation service health in the given cluster. This method
        was added in vSphere API 7.0.0.0.

        :type  cluster: :class:`str`
        :param cluster: Identifier of the cluster.
            The parameter must be an identifier for the resource type:
            ``ClusterComputeResource``.
        :rtype: :class:  `vmware.vapi.stdlib.client.task.Task`
        :return: Task instance
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            For any other error.
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidArgument` 
            If the cluster id is empty.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            If the cluster is not found.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            If the caller is not authenticated.
        """
        task_id = self._invoke('get$task',
                                {
                                'cluster': cluster,
                                })
        task_svc = Tasks(self._config)
        task_instance = Task(task_id, task_svc, type.ReferenceType(__name__, 'ServiceStatus.Info'))
        return task_instance
class _ServiceStatusStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'cluster': type.IdType(resource_types='ClusterComputeResource'),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),
            'com.vmware.vapi.std.errors.invalid_argument':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidArgument'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/vcenter/trusted-infrastructure/trust-authority-clusters/{cluster}/attestation/service-status',
            path_variables={
                'cluster': 'cluster',
            },
            query_parameters={
            },
            dispatch_parameters={
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        operations = {
            'get$task': {
                'input_type': get_input_type,
                'output_type': type.IdType(resource_types='com.vmware.cis.TASK'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': [],
                'task_type': TaskType.TASK_ONLY,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.attestation.service_status',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=True)


class StubFactory(StubFactoryBase):
    _attrs = {
        'ServiceStatus': ServiceStatus,
        'tpm2': 'com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.attestation.tpm2_client.StubFactory',
    }




"""
Utility library for Provider modules
"""

__author__ = 'VMware, Inc.'
__copyright__ = 'Copyright 2015 VMware, Inc.  All rights reserved. -- VMware Confidential'  # pylint: disable=line-too-long

from vmware.vapi.core import MethodResult, MethodDefinition
from vmware.vapi.data.value import StructValue, ListValue
from vmware.vapi.lib.constants import Introspection


def augment_method_result_with_errors(service_id, operation_id, method_result,
                                      errors_to_augment):
    """
    Returns a new method result that is identical to `method_result` except that
    the `errors_definition` field in the `output` (which is of type
    Operation.Info from Introspection service) contains the errors from the Info
    structure in `method_result` plus the errors in `errors_to_augment`. This
    code will be executed only for "get" operation in vAPI Operation
    Introspection service.

    :type  service_id: :class:`str`
    :param service_id: Service identifier
    :type  operation_id: :class:`str`
    :param operation_id: Operation identifier
    :type  method_result: :class:`vmware.vapi.core.MethodResult`
    :param method_result: Operation result
    :type  errors_to_augment: :class:`list` of
        :class:`vmware.vapi.data.value.StructValue`
    :param errors_to_augment: Errors to augment. These are struct values of type
        com.vmware.vapi.std.introspection.Operation.DataDefinition whose `type`
        field has the value ERROR
        to the DataDefinition type in Introspection service IDL.
    :rtype: :class:`vmware.vapi.data.value.DataValue`
    :return: Output data value
    """
    if method_result.success():
        if (service_id == Introspection.OPERATION_SVC and
                operation_id == 'get'):
            output = method_result.output
            augmented_output = StructValue(
                'com.vmware.vapi.std.introspection.operation.info')
            augmented_output.set_field(
                'input_definition',
                output.get_field('input_definition'))
            augmented_output.set_field(
                'output_definition',
                output.get_field('output_definition'))
            errors = ListValue()
            error_names = []
            for error_def in output.get_field('error_definitions'):
                errors.add(error_def)
                error_names.append(error_def.get_field('name').value.value)
            for error_def in errors_to_augment:
                if error_def.get_field('name').value.value not in error_names:
                    errors.add(error_def)
            augmented_output.set_field('error_definitions', errors)
            return MethodResult(output=augmented_output)
    return method_result


def augment_method_def_with_errors(method_def, errors):
    """
    Add errors reported by this ApiProviderFilter to a method definition.
    This method clones the input method definition and appends errors. It
    does not modify the parameter.

    :type  method_def: :class:`vmware.vapi.core.MethodDefinition`
    :param method_def: Method definition
    :type  errors: :class:`list` of
        :class:`vmware.vapi.data.definition.ErrorDefinition`
    :param errors: Error definitions to augment to MethodDefinition
    :rtype: :class:`vmware.vapi.core.MethodDefinition`
    :return: Method definition after augmenting the errors
    """
    method_errors = method_def.get_error_definitions()
    return MethodDefinition(method_def.get_identifier(),
                            method_def.get_input_definition(),
                            method_def.get_output_definition(),
                            method_errors.union(errors))



"""
String Constants used in vAPI runtime
"""

__author__ = 'VMware, Inc.'
__copyright__ = 'Copyright 2015-2019 VMware, Inc.  All rights reserved. -- VMware Confidential'  # pylint: disable=line-too-long


PARAMS = 'params'
SCHEME_ID = 'schemeId'
AUTHN_IDENTITY = 'authnIdentity'
EXECUTION_CONTEXT = 'ctx'
APPLICATION_CONTEXT = 'appCtx'
SECURITY_CONTEXT = 'securityCtx'
PROCESSORS = 'processors'
OPID = 'opId'
SHOW_UNRELEASED_APIS = '$showUnreleasedAPIs'
TASK_OPERATION = '$task'
TASK_ID = '$taskId'
TASK_REST_QUERY_PARAM = 'vmw-task'
REST_OP_ID_HEADER = 'X-Request-ID'

# Magic structure names
# Structure name for the StructValues that represent
# map entries in the runtime
MAP_ENTRY = 'map-entry'
# Structure name for the StructValue that represent
# maps when using cleanjson
MAP_STRUCT = 'map-struct'
# Structure name for the StructValues that represent
# operation input in the runtime
OPERATION_INPUT = 'operation-input'

# Structure name for the StructValue that represent
# a dynamic structure in the absence of the the type name
DYNAMIC_STRUCTURE = 'dynamic-structure'

# Constants for REST presentation Layer
JSONRPC = 'jsonrpc'
JSON_CONTENT_TYPE = 'application/json'
HTTP_ACCEPT_HEADER = 'Accept'
HTTP_USER_AGENT_HEADER = 'User-Agent'
HTTP_CONTENT_TYPE_HEADER = 'Content-Type'
HTTP_ACCEPT_LANGUAGE = 'accept-language'
HTTP_FORMAT_LOCALE = 'format-locale'
HTTP_TIMEZONE = 'timezone'
LOCALE = 'locale'

# HTTP headers
VAPI_ERROR_HEADER = 'vapi-error'
VAPI_HEADER_PREFIX = 'vapi-ctx-'
VAPI_SERVICE_HEADER = 'vapi-service'
VAPI_OPERATION_HEADER = 'vapi-operation'
VAPI_SESSION_HEADER = 'vmware-api-session-id'

header_mapping_dict = {
    'opid': 'opId',
    'actid': 'actId',
    '$showunreleasedapis': '$showUnreleasedAPIs',
    '$useragent': '$userAgent',
    '$donotroute': '$doNotRoute',
    'vmwaresessionid': 'vmwareSessionId',
    'activationid': 'ActivationId',
    '$taskid': '$taskId',
}


class Introspection(object):
    """
    String constants used in introsection service
    """
    PACKAGE = 'com.vmware.vapi.std.introspection'

    # Services
    PROVIDER_SVC = 'com.vmware.vapi.std.introspection.provider'
    SERVICE_SVC = 'com.vmware.vapi.std.introspection.service'
    OPERATION_SVC = 'com.vmware.vapi.std.introspection.operation'

    # Types
    DATA_DEFINITION = \
        'com.vmware.vapi.std.introspection.operation.data_definition'


class RestAnnotations(object):
    """
    String constants used in REST annotations in VMODL definition
    """
    REQUEST_MAPPING = 'RequestMapping'
    PATH_VARIABLE = 'PathVariable'
    QUERY_VARIABLE = 'Query'
    METHOD_ELEMENT = 'method'
    VALUE_ELEMENT = 'value'
    ACTION_PARAM = 'action'
    PARAMS_ELEMENT = 'params'
    PATH_ELEMENT = 'path'
    HEADERS_ELEMENT = 'headers'
    NAME_ELEMENT = 'name'
    RESPONSE_ELEMENT = 'Response'
    HEADER_ELEMENT = 'Header'
    BODY_ELEMENT = 'Body'
    VERB_GET = 'GET'
    VERB_POST = 'POST'
    VERB_PUT = 'PUT'
    VERB_PATCH = 'PATCH'
    VERB_DELETE = 'DELETE'


class RestAnnotationType(object):
    """
    Rest annotation type in VMODL definition
    """
    NONE = 0
    REQUEST = 1
    VERB = 2


class TaskType(object):
    """
    Task types
    """
    NONE = 0
    TASK = 1
    TASK_ONLY = 2



# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.nsx_policy.infra.tags.
#---------------------------------------------------------------------------

"""


"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class EffectiveResources(VapiInterface):
    """
    
    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx_policy.infra.tags.effective_resources'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _EffectiveResourcesStub)
        self._VAPI_OPERATION_IDS = {}


    def list(self,
             cursor=None,
             enforcement_point_path=None,
             filter_text=None,
             include_mark_for_delete_objects=None,
             included_fields=None,
             page_size=None,
             scope=None,
             sort_ascending=None,
             sort_by=None,
             tag=None,
             ):
        """
        Paginated list of all objects assigned with matching scope and tag
        values. Objects are represented in form of resource reference. Sort
        option is available only on target_type and target_display_name
        properties.

        :type  cursor: :class:`str` or ``None``
        :param cursor: Opaque cursor to be used for getting next page of records (supplied
            by current result page) (optional)
        :type  enforcement_point_path: :class:`str` or ``None``
        :param enforcement_point_path: String Path of the enforcement point (optional)
        :type  filter_text: :class:`str` or ``None``
        :param filter_text: Filter text to restrict tagged objects list with matching filter
            text. (optional)
        :type  include_mark_for_delete_objects: :class:`bool` or ``None``
        :param include_mark_for_delete_objects: Include objects that are marked for deletion in results (optional,
            default to false)
        :type  included_fields: :class:`str` or ``None``
        :param included_fields: Comma separated list of fields that should be included in query
            result (optional)
        :type  page_size: :class:`long` or ``None``
        :param page_size: Maximum number of results to return in this page (server may return
            fewer) (optional, default to 1000)
        :type  scope: :class:`str` or ``None``
        :param scope: Tag scope (optional)
        :type  sort_ascending: :class:`bool` or ``None``
        :param sort_ascending: (optional)
        :type  sort_by: :class:`str` or ``None``
        :param sort_by: Field by which records are sorted (optional)
        :type  tag: :class:`str` or ``None``
        :param tag: Tag value (optional)
        :rtype: :class:`com.vmware.nsx_policy.model_client.PolicyResourceReferenceListResult`
        :return: com.vmware.nsx_policy.model.PolicyResourceReferenceListResult
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('list',
                            {
                            'cursor': cursor,
                            'enforcement_point_path': enforcement_point_path,
                            'filter_text': filter_text,
                            'include_mark_for_delete_objects': include_mark_for_delete_objects,
                            'included_fields': included_fields,
                            'page_size': page_size,
                            'scope': scope,
                            'sort_ascending': sort_ascending,
                            'sort_by': sort_by,
                            'tag': tag,
                            })
class TagOperations(VapiInterface):
    """
    
    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx_policy.infra.tags.tag_operations'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _TagOperationsStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            operation_id,
            cursor=None,
            enforcement_point_path=None,
            include_mark_for_delete_objects=None,
            included_fields=None,
            page_size=None,
            sort_ascending=None,
            sort_by=None,
            ):
        """
        Get details of tag bulk operation request with which tag is applied or
        removed on virtual machines.

        :type  operation_id: :class:`str`
        :param operation_id: (required)
        :type  cursor: :class:`str` or ``None``
        :param cursor: Opaque cursor to be used for getting next page of records (supplied
            by current result page) (optional)
        :type  enforcement_point_path: :class:`str` or ``None``
        :param enforcement_point_path: String Path of the enforcement point (optional)
        :type  include_mark_for_delete_objects: :class:`bool` or ``None``
        :param include_mark_for_delete_objects: Include objects that are marked for deletion in results (optional,
            default to false)
        :type  included_fields: :class:`str` or ``None``
        :param included_fields: Comma separated list of fields that should be included in query
            result (optional)
        :type  page_size: :class:`long` or ``None``
        :param page_size: Maximum number of results to return in this page (server may return
            fewer) (optional, default to 1000)
        :type  sort_ascending: :class:`bool` or ``None``
        :param sort_ascending: (optional)
        :type  sort_by: :class:`str` or ``None``
        :param sort_by: Field by which records are sorted (optional)
        :rtype: :class:`com.vmware.nsx_policy.model_client.TagBulkOperation`
        :return: com.vmware.nsx_policy.model.TagBulkOperation
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('get',
                            {
                            'operation_id': operation_id,
                            'cursor': cursor,
                            'enforcement_point_path': enforcement_point_path,
                            'include_mark_for_delete_objects': include_mark_for_delete_objects,
                            'included_fields': included_fields,
                            'page_size': page_size,
                            'sort_ascending': sort_ascending,
                            'sort_by': sort_by,
                            })

    def update(self,
               operation_id,
               tag_bulk_operation,
               cursor=None,
               enforcement_point_path=None,
               include_mark_for_delete_objects=None,
               included_fields=None,
               page_size=None,
               sort_ascending=None,
               sort_by=None,
               ):
        """
        Tag can be assigned or unassigned on multiple objects. Supported object
        type is restricted to Virtual Machine for now and support for other
        objects will be added later. Permissions for tag bulk operation would
        be similar to virtual machine tag permissions.

        :type  operation_id: :class:`str`
        :param operation_id: (required)
        :type  tag_bulk_operation: :class:`com.vmware.nsx_policy.model_client.TagBulkOperation`
        :param tag_bulk_operation: (required)
        :type  cursor: :class:`str` or ``None``
        :param cursor: Opaque cursor to be used for getting next page of records (supplied
            by current result page) (optional)
        :type  enforcement_point_path: :class:`str` or ``None``
        :param enforcement_point_path: String Path of the enforcement point (optional)
        :type  include_mark_for_delete_objects: :class:`bool` or ``None``
        :param include_mark_for_delete_objects: Include objects that are marked for deletion in results (optional,
            default to false)
        :type  included_fields: :class:`str` or ``None``
        :param included_fields: Comma separated list of fields that should be included in query
            result (optional)
        :type  page_size: :class:`long` or ``None``
        :param page_size: Maximum number of results to return in this page (server may return
            fewer) (optional, default to 1000)
        :type  sort_ascending: :class:`bool` or ``None``
        :param sort_ascending: (optional)
        :type  sort_by: :class:`str` or ``None``
        :param sort_by: Field by which records are sorted (optional)
        :rtype: :class:`com.vmware.nsx_policy.model_client.TagBulkOperation`
        :return: com.vmware.nsx_policy.model.TagBulkOperation
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('update',
                            {
                            'operation_id': operation_id,
                            'tag_bulk_operation': tag_bulk_operation,
                            'cursor': cursor,
                            'enforcement_point_path': enforcement_point_path,
                            'include_mark_for_delete_objects': include_mark_for_delete_objects,
                            'included_fields': included_fields,
                            'page_size': page_size,
                            'sort_ascending': sort_ascending,
                            'sort_by': sort_by,
                            })
class _EffectiveResourcesStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for list operation
        list_input_type = type.StructType('operation-input', {
            'cursor': type.OptionalType(type.StringType()),
            'enforcement_point_path': type.OptionalType(type.StringType()),
            'filter_text': type.OptionalType(type.StringType()),
            'include_mark_for_delete_objects': type.OptionalType(type.BooleanType()),
            'included_fields': type.OptionalType(type.StringType()),
            'page_size': type.OptionalType(type.IntegerType()),
            'scope': type.OptionalType(type.StringType()),
            'sort_ascending': type.OptionalType(type.BooleanType()),
            'sort_by': type.OptionalType(type.StringType()),
            'tag': type.OptionalType(type.StringType()),
        })
        list_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        list_input_value_validator_list = [
        ]
        list_output_validator_list = [
        ]
        list_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/policy/api/v1/infra/tags/effective-resources',
            path_variables={
            },
             header_parameters={
                                 },
            query_parameters={
                'cursor': 'cursor',
                'enforcement_point_path': 'enforcement_point_path',
                'filter_text': 'filter_text',
                'include_mark_for_delete_objects': 'include_mark_for_delete_objects',
                'included_fields': 'included_fields',
                'page_size': 'page_size',
                'scope': 'scope',
                'sort_ascending': 'sort_ascending',
                'sort_by': 'sort_by',
                'tag': 'tag',
            },
            content_type='application/json'
        )

        operations = {
            'list': {
                'input_type': list_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx_policy.model_client', 'PolicyResourceReferenceListResult'),
                'errors': list_error_dict,
                'input_value_validator_list': list_input_value_validator_list,
                'output_validator_list': list_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'list': list_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx_policy.infra.tags.effective_resources',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _TagOperationsStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'operation_id': type.StringType(),
            'cursor': type.OptionalType(type.StringType()),
            'enforcement_point_path': type.OptionalType(type.StringType()),
            'include_mark_for_delete_objects': type.OptionalType(type.BooleanType()),
            'included_fields': type.OptionalType(type.StringType()),
            'page_size': type.OptionalType(type.IntegerType()),
            'sort_ascending': type.OptionalType(type.BooleanType()),
            'sort_by': type.OptionalType(type.StringType()),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
            HasFieldsOfValidator()
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/policy/api/v1/infra/tags/tag-operations/{operation-id}',
            path_variables={
                'operation_id': 'operation-id',
            },
             header_parameters={
                             },
            query_parameters={
                'cursor': 'cursor',
                'enforcement_point_path': 'enforcement_point_path',
                'include_mark_for_delete_objects': 'include_mark_for_delete_objects',
                'included_fields': 'included_fields',
                'page_size': 'page_size',
                'sort_ascending': 'sort_ascending',
                'sort_by': 'sort_by',
            },
            content_type='application/json'
        )

        # properties for update operation
        update_input_type = type.StructType('operation-input', {
            'operation_id': type.StringType(),
            'tag_bulk_operation': type.ReferenceType('com.vmware.nsx_policy.model_client', 'TagBulkOperation'),
            'cursor': type.OptionalType(type.StringType()),
            'enforcement_point_path': type.OptionalType(type.StringType()),
            'include_mark_for_delete_objects': type.OptionalType(type.BooleanType()),
            'included_fields': type.OptionalType(type.StringType()),
            'page_size': type.OptionalType(type.IntegerType()),
            'sort_ascending': type.OptionalType(type.BooleanType()),
            'sort_by': type.OptionalType(type.StringType()),
        })
        update_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        update_input_value_validator_list = [
            HasFieldsOfValidator()
        ]
        update_output_validator_list = [
            HasFieldsOfValidator()
        ]
        update_rest_metadata = OperationRestMetadata(
            http_method='PUT',
            url_template='/policy/api/v1/infra/tags/tag-operations/{operation-id}',
            request_body_parameter='tag_bulk_operation',
            path_variables={
                'operation_id': 'operation-id',
            },
             header_parameters={
                               },
            query_parameters={
                'cursor': 'cursor',
                'enforcement_point_path': 'enforcement_point_path',
                'include_mark_for_delete_objects': 'include_mark_for_delete_objects',
                'included_fields': 'included_fields',
                'page_size': 'page_size',
                'sort_ascending': 'sort_ascending',
                'sort_by': 'sort_by',
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx_policy.model_client', 'TagBulkOperation'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'update': {
                'input_type': update_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx_policy.model_client', 'TagBulkOperation'),
                'errors': update_error_dict,
                'input_value_validator_list': update_input_value_validator_list,
                'output_validator_list': update_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
            'update': update_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx_policy.infra.tags.tag_operations',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)


class StubFactory(StubFactoryBase):
    _attrs = {
        'EffectiveResources': EffectiveResources,
        'TagOperations': TagOperations,
        'tag_operations': 'com.vmware.nsx_policy.infra.tags.tag_operations_client.StubFactory',
    }




# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.vcenter.vm.guest.networking.
#---------------------------------------------------------------------------

"""
The ``com.vmware.vcenter.vm.guest.networking_client`` module provides classes
for dealing with the guest operating system networking.

"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class Interfaces(VapiInterface):
    """
    The ``Interfaces`` class provides methods for retrieving guest operating
    system network interface information. This class was added in vSphere API
    7.0.0.0.
    """

    _VAPI_SERVICE_ID = 'com.vmware.vcenter.vm.guest.networking.interfaces'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _InterfacesStub)
        self._VAPI_OPERATION_IDS = {}

    class IpAddressOrigin(Enum):
        """
        The ``Interfaces.IpAddressOrigin`` class specifies how an IP address was
        obtained for an interface. See RFC 4293 IpAddressOriginTC. This enumeration
        was added in vSphere API 7.0.0.0.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        OTHER = None
        """
        Any other type of address configuration other than the below mentioned ones
        will fall under this category. For e.g., automatic address configuration
        for the link local address falls under this type. This class attribute was
        added in vSphere API 7.0.0.0.

        """
        MANUAL = None
        """
        The address is configured manually. This class attribute was added in
        vSphere API 7.0.0.0.

        """
        DHCP = None
        """
        The address is configured through dhcp. This class attribute was added in
        vSphere API 7.0.0.0.

        """
        LINKLAYER = None
        """
        The address is obtained through stateless autoconfiguration (autoconf). See
        RFC 4862, IPv6 Stateless Address Autoconfiguration. This class attribute
        was added in vSphere API 7.0.0.0.

        """
        RANDOM = None
        """
        The address is chosen by the system at random e.g., an IPv4 address within
        169.254/16, or an RFC 3041 privacy address. This class attribute was added
        in vSphere API 7.0.0.0.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`IpAddressOrigin` instance.
            """
            Enum.__init__(string)

    IpAddressOrigin._set_values({
        'OTHER': IpAddressOrigin('OTHER'),
        'MANUAL': IpAddressOrigin('MANUAL'),
        'DHCP': IpAddressOrigin('DHCP'),
        'LINKLAYER': IpAddressOrigin('LINKLAYER'),
        'RANDOM': IpAddressOrigin('RANDOM'),
    })
    IpAddressOrigin._set_binding_type(type.EnumType(
        'com.vmware.vcenter.vm.guest.networking.interfaces.ip_address_origin',
        IpAddressOrigin))


    class IpAddressStatus(Enum):
        """
        The ``Interfaces.IpAddressStatus`` class defines the present status of an
        address on an interface. See RFC 4293 IpAddressStatusTC. This enumeration
        was added in vSphere API 7.0.0.0.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        PREFERRED = None
        """
        Indicates that this is a valid address. This class attribute was added in
        vSphere API 7.0.0.0.

        """
        DEPRECATED = None
        """
        Indicates that this is a valid but deprecated address that should no longer
        be used as a source address. This class attribute was added in vSphere API
        7.0.0.0.

        """
        INVALID = None
        """
        Indicates that this isn't a valid address. This class attribute was added
        in vSphere API 7.0.0.0.

        """
        INACCESSIBLE = None
        """
        Indicates that the address is not accessible because interface is not
        operational. This class attribute was added in vSphere API 7.0.0.0.

        """
        UNKNOWN = None
        """
        Indicates that the status cannot be determined. This class attribute was
        added in vSphere API 7.0.0.0.

        """
        TENTATIVE = None
        """
        Indicates that the uniqueness of the address on the link is presently being
        verified. This class attribute was added in vSphere API 7.0.0.0.

        """
        DUPLICATE = None
        """
        Indicates the address has been determined to be non-unique on the link,
        this address will not be reachable. This class attribute was added in
        vSphere API 7.0.0.0.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`IpAddressStatus` instance.
            """
            Enum.__init__(string)

    IpAddressStatus._set_values({
        'PREFERRED': IpAddressStatus('PREFERRED'),
        'DEPRECATED': IpAddressStatus('DEPRECATED'),
        'INVALID': IpAddressStatus('INVALID'),
        'INACCESSIBLE': IpAddressStatus('INACCESSIBLE'),
        'UNKNOWN': IpAddressStatus('UNKNOWN'),
        'TENTATIVE': IpAddressStatus('TENTATIVE'),
        'DUPLICATE': IpAddressStatus('DUPLICATE'),
    })
    IpAddressStatus._set_binding_type(type.EnumType(
        'com.vmware.vcenter.vm.guest.networking.interfaces.ip_address_status',
        IpAddressStatus))


    class IpAddressInfo(VapiStruct):
        """
        The ``Interfaces.IpAddressInfo`` class describes a specific IP Address.
        This class was added in vSphere API 7.0.0.0.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     ip_address=None,
                     prefix_length=None,
                     origin=None,
                     state=None,
                    ):
            """
            :type  ip_address: :class:`str`
            :param ip_address: IPv4 address is specified using dotted decimal notation. For
                example, "192.0.2.1". IPv6 addresses are 128-bit addresses
                specified using eight fields of up to four hexadecimal digits. A
                colon separates each field (:). For example,
                2001:DB8:101::230:6eff:fe04:d9ff. The address can also consist of
                the symbol '::' to represent multiple 16-bit groups of contiguous
                0's only once in an address as described in RFC 2373. This
                attribute was added in vSphere API 7.0.0.0.
            :type  prefix_length: :class:`long`
            :param prefix_length: Denotes the length of a generic Internet network address prefix.
                Prefix length: the valid range of values is 0-32 for IPv4, and
                0-128 for IPv6. A value of n corresponds to an IP address mask that
                has n contiguous 1-bits from the most significant bit (MSB), with
                all other bits set to 0. A value of zero is valid only if the
                calling context defines it. This attribute was added in vSphere API
                7.0.0.0.
            :type  origin: :class:`Interfaces.IpAddressOrigin` or ``None``
            :param origin: How this address was configured. This attribute was added in
                vSphere API 7.0.0.0.
                If None the data was not available.
            :type  state: :class:`Interfaces.IpAddressStatus`
            :param state: The state of this ipAddress. This attribute was added in vSphere
                API 7.0.0.0.
            """
            self.ip_address = ip_address
            self.prefix_length = prefix_length
            self.origin = origin
            self.state = state
            VapiStruct.__init__(self)


    IpAddressInfo._set_binding_type(type.StructType(
        'com.vmware.vcenter.vm.guest.networking.interfaces.ip_address_info', {
            'ip_address': type.StringType(),
            'prefix_length': type.IntegerType(),
            'origin': type.OptionalType(type.ReferenceType(__name__, 'Interfaces.IpAddressOrigin')),
            'state': type.ReferenceType(__name__, 'Interfaces.IpAddressStatus'),
        },
        IpAddressInfo,
        False,
        None))


    class IpConfigInfo(VapiStruct):
        """
        The ``Interfaces.IpConfigInfo`` class describes the protocol version
        independent address reporting data object for network interfaces. This
        class was added in vSphere API 7.0.0.0.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     ip_addresses=None,
                     dhcp=None,
                    ):
            """
            :type  ip_addresses: :class:`list` of :class:`Interfaces.IpAddressInfo`
            :param ip_addresses: IP addresses configured on the interface. This attribute was added
                in vSphere API 7.0.0.0.
            :type  dhcp: :class:`com.vmware.vcenter.vm.guest_client.DhcpConfigInfo` or ``None``
            :param dhcp: Client side DHCP for an interface. This attribute was added in
                vSphere API 7.0.0.0.
                If None the IP was not configured by DHCP.
            """
            self.ip_addresses = ip_addresses
            self.dhcp = dhcp
            VapiStruct.__init__(self)


    IpConfigInfo._set_binding_type(type.StructType(
        'com.vmware.vcenter.vm.guest.networking.interfaces.ip_config_info', {
            'ip_addresses': type.ListType(type.ReferenceType(__name__, 'Interfaces.IpAddressInfo')),
            'dhcp': type.OptionalType(type.ReferenceType('com.vmware.vcenter.vm.guest_client', 'DhcpConfigInfo')),
        },
        IpConfigInfo,
        False,
        None))


    class Info(VapiStruct):
        """
        The ``Interfaces.Info`` class describes a virtual network adapter
        configured in the guest operating system. This class was added in vSphere
        API 7.0.0.0.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     dns_values=None,
                     mac_address=None,
                     dns=None,
                     ip=None,
                     wins_servers=None,
                     nic=None,
                    ):
            """
            :type  dns_values: :class:`com.vmware.vcenter.vm.guest_client.DnsAssignedValues` or ``None``
            :param dns_values: Client DNS values. Data assigned by DNS. This attribute was added
                in vSphere API 7.0.0.0.
                If None no DNS assigned value exists.
            :type  mac_address: :class:`str` or ``None``
            :param mac_address: MAC address of the adapter. This attribute was added in vSphere API
                7.0.0.0.
                If None then not supported by the Guest OS.
            :type  dns: :class:`com.vmware.vcenter.vm.guest_client.DnsConfigInfo` or ``None``
            :param dns: DNS configuration of the adapter. See
                :attr:`com.vmware.vcenter.vm.guest_client.Networking.Info.dns` for
                system wide settings. This attribute was added in vSphere API
                7.0.0.0.
                If None then not assigned by the Guest OS.
            :type  ip: :class:`Interfaces.IpConfigInfo` or ``None``
            :param ip: IP configuration settings of the adapter. This attribute was added
                in vSphere API 7.0.0.0.
                If None then not supported by the Guest OS.
            :type  wins_servers: :class:`list` of :class:`str` or ``None``
            :param wins_servers: The IP addresses of any WINS name servers for the adapter. This
                attribute was added in vSphere API 7.0.0.0.
                If None then not supported by the Guest OS.
            :type  nic: :class:`str` or ``None``
            :param nic: Link to the corresponding virtual device. This attribute was added
                in vSphere API 7.0.0.0.
                When clients pass a value of this class as a parameter, the
                attribute must be an identifier for the resource type:
                ``com.vmware.vcenter.vm.hardware.Ethernet``. When methods return a
                value of this class as a return value, the attribute will be an
                identifier for the resource type:
                ``com.vmware.vcenter.vm.hardware.Ethernet``.
                If None then the interface is not backed by a virtual device.
            """
            self.dns_values = dns_values
            self.mac_address = mac_address
            self.dns = dns
            self.ip = ip
            self.wins_servers = wins_servers
            self.nic = nic
            VapiStruct.__init__(self)


    Info._set_binding_type(type.StructType(
        'com.vmware.vcenter.vm.guest.networking.interfaces.info', {
            'dns_values': type.OptionalType(type.ReferenceType('com.vmware.vcenter.vm.guest_client', 'DnsAssignedValues')),
            'mac_address': type.OptionalType(type.StringType()),
            'dns': type.OptionalType(type.ReferenceType('com.vmware.vcenter.vm.guest_client', 'DnsConfigInfo')),
            'ip': type.OptionalType(type.ReferenceType(__name__, 'Interfaces.IpConfigInfo')),
            'wins_servers': type.OptionalType(type.ListType(type.StringType())),
            'nic': type.OptionalType(type.IdType()),
        },
        Info,
        False,
        None))



    def list(self,
             vm,
             ):
        """
        Returns information about the networking interfaces in the guest
        operating system. This method was added in vSphere API 7.0.0.0.

        :type  vm: :class:`str`
        :param vm: Virtual machine ID
            The parameter must be an identifier for the resource type:
            ``VirtualMachine``.
        :rtype: :class:`list` of :class:`Interfaces.Info`
        :return: Information about the interfaces configured in the guest operating
            system. Interfaces are ordered in a guest operating system specific
            determined order.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            if the system reports an error while responding to the request.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            if the virtual machine is not found.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            if VMware Tools is not running.
        """
        return self._invoke('list',
                            {
                            'vm': vm,
                            })
class Routes(VapiInterface):
    """
    The ``Routes`` class provides methods for retrieving guest operating system
    network routing information. This class was added in vSphere API 7.0.0.0.
    """

    _VAPI_SERVICE_ID = 'com.vmware.vcenter.vm.guest.networking.routes'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _RoutesStub)
        self._VAPI_OPERATION_IDS = {}

    class Info(VapiStruct):
        """
        The ``Routes.Info`` class describes an individual host, network or default
        destination network reachable through a gateway. This class was added in
        vSphere API 7.0.0.0.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     network=None,
                     prefix_length=None,
                     gateway_address=None,
                     interface_index=None,
                    ):
            """
            :type  network: :class:`str`
            :param network: IP Address of the destination IP network. IPv4 address is specified
                using dotted decimal notation. For example, "192.0.2.1". IPv6
                addresses are 128-bit specified using as eight fields of up to four
                hexadecimal digits. A colon separates each field (:). For example,
                2001:DB8:101::230:6eff:fe04:d9ff. The address can also consist of
                symbol '::' to represent multiple 16-bit groups of contiguous 0's
                only once in an address as described in RFC 2373. This attribute
                was added in vSphere API 7.0.0.0.
            :type  prefix_length: :class:`long`
            :param prefix_length: The prefix length. For IPv4 the value range is 0-32. For IPv6
                prefixLength is a decimal value range 0-128. The property
                represents the number of contiguous, higher-order bits of the
                address that make up the network portion of the IP address. This
                attribute was added in vSphere API 7.0.0.0.
            :type  gateway_address: :class:`str` or ``None``
            :param gateway_address: Where to send the packets for this route. Unicast IP Address of the
                next hop router. IPv4 address is specified using dotted decimal
                notation. For example, "192.0.2.1". IPv6 addresses are 128-bit
                specified using as eight fields of up to four hexadecimal digits. A
                colon separates each field (:). For example,
                2001:DB8:101::230:6eff:fe04:d9ff. The address can also consist of
                symbol '::' to represent multiple 16-bit groups of contiguous 0's
                only once in an address as described in RFC 2373. This attribute
                was added in vSphere API 7.0.0.0.
                If None no gateway is set for the route.
            :type  interface_index: :class:`long` or ``None``
            :param interface_index: The network interface associated with this route. This is an index
                into the result of :func:`Interfaces.list` The index refers to the
                relative position of an element in a :class:`list`. For example, an
                index of 0 refers to the first element in the :class:`list` while
                an index of 1 refers to the second element. This attribute was
                added in vSphere API 7.0.0.0.
                If None the route is not associated with a network interface.
            """
            self.network = network
            self.prefix_length = prefix_length
            self.gateway_address = gateway_address
            self.interface_index = interface_index
            VapiStruct.__init__(self)


    Info._set_binding_type(type.StructType(
        'com.vmware.vcenter.vm.guest.networking.routes.info', {
            'network': type.StringType(),
            'prefix_length': type.IntegerType(),
            'gateway_address': type.OptionalType(type.StringType()),
            'interface_index': type.OptionalType(type.IntegerType()),
        },
        Info,
        False,
        None))



    def list(self,
             vm,
             ):
        """
        Returns information about network routing in the guest operating
        system. This method was added in vSphere API 7.0.0.0.

        :type  vm: :class:`str`
        :param vm: Virtual machine ID
            The parameter must be an identifier for the resource type:
            ``VirtualMachine``.
        :rtype: :class:`list` of :class:`Routes.Info`
        :return: Information about the network routes configured in the guest
            operating system.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            if the system reports an error while responding to the request.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            if the virtual machine is not found.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            if VMware Tools is not running.
        """
        return self._invoke('list',
                            {
                            'vm': vm,
                            })
class _InterfacesStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for list operation
        list_input_type = type.StructType('operation-input', {
            'vm': type.IdType(resource_types='VirtualMachine'),
        })
        list_error_dict = {
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),

        }
        list_input_value_validator_list = [
        ]
        list_output_validator_list = [
        ]
        list_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/vcenter/vm/{vm}/guest/networking/interfaces',
            path_variables={
                'vm': 'vm',
            },
             header_parameters={
               },
            query_parameters={
            }
        )

        operations = {
            'list': {
                'input_type': list_input_type,
                'output_type': type.ListType(type.ReferenceType(__name__, 'Interfaces.Info')),
                'errors': list_error_dict,
                'input_value_validator_list': list_input_value_validator_list,
                'output_validator_list': list_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'list': list_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vcenter.vm.guest.networking.interfaces',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=True)

class _RoutesStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for list operation
        list_input_type = type.StructType('operation-input', {
            'vm': type.IdType(resource_types='VirtualMachine'),
        })
        list_error_dict = {
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),

        }
        list_input_value_validator_list = [
        ]
        list_output_validator_list = [
        ]
        list_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/vcenter/vm/{vm}/guest/networking/routes',
            path_variables={
                'vm': 'vm',
            },
             header_parameters={
               },
            query_parameters={
            }
        )

        operations = {
            'list': {
                'input_type': list_input_type,
                'output_type': type.ListType(type.ReferenceType(__name__, 'Routes.Info')),
                'errors': list_error_dict,
                'input_value_validator_list': list_input_value_validator_list,
                'output_validator_list': list_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'list': list_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vcenter.vm.guest.networking.routes',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=True)


class StubFactory(StubFactoryBase):
    _attrs = {
        'Interfaces': Interfaces,
        'Routes': Routes,
    }




# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.vcenter.namespace_management.supervisors.networks.management.
#---------------------------------------------------------------------------

"""


"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata

class NetworkBackingEnum(Enum):
    """
    
    
    ``NetworkBackingEnum`` enumerates types of network backings supported by
    the Supervisor for the management network.. This enumeration was added in
    vSphere API 8.0.0.1.

    .. note::
        This class represents an enumerated type in the interface language
        definition. The class contains class attributes which represent the
        values in the current version of the enumerated type. Newer versions of
        the enumerated type may contain new values. To use new values of the
        enumerated type in communication with a server that supports the newer
        version of the API, you instantiate this class. See :ref:`enumerated
        type description page <enumeration_description>`.
    """
    NETWORK = None
    """
    
    
    Indicates a virtual infrastructure management Network. It may include items
    like opaque networks or Distributed Virtual Port Groups.. This class
    attribute was added in vSphere API 8.0.0.1.

    """

    def __init__(self, string):
        """
        :type  string: :class:`str`
        :param string: String value for the :class:`NetworkBackingEnum` instance.
        """
        Enum.__init__(string)

NetworkBackingEnum._set_values({
    'NETWORK': NetworkBackingEnum('NETWORK'),
})
NetworkBackingEnum._set_binding_type(type.EnumType(
    'com.vmware.vcenter.namespace_management.supervisors.networks.management.network_backing_enum',
    NetworkBackingEnum))



class ProxySettingsSource(Enum):
    """
    
    
    The settings can be inherited from the vCenter settings, so the Supervisor
    settings will be synced. The settings can be applied directly on the
    Supervisor level, or the Supervisor can be configured not to use a proxy..
    This enumeration was added in vSphere API 8.0.0.1.

    .. note::
        This class represents an enumerated type in the interface language
        definition. The class contains class attributes which represent the
        values in the current version of the enumerated type. Newer versions of
        the enumerated type may contain new values. To use new values of the
        enumerated type in communication with a server that supports the newer
        version of the API, you instantiate this class. See :ref:`enumerated
        type description page <enumeration_description>`.
    """
    VC_INHERITED = None
    """
    
    
    Proxy settings will be inherited from the vCenter settings. vCenter and
    Supervisor settings will be kept in sync.. This class attribute was added
    in vSphere API 8.0.0.1.

    """
    CLUSTER_CONFIGURED = None
    """
    Proxy settings will be configured at the Supervisor level. This class
    attribute was added in vSphere API 8.0.0.1.

    """
    NONE = None
    """
    No proxy settings will be applied to the Supervisor. This class attribute
    was added in vSphere API 8.0.0.1.

    """

    def __init__(self, string):
        """
        :type  string: :class:`str`
        :param string: String value for the :class:`ProxySettingsSource` instance.
        """
        Enum.__init__(string)

ProxySettingsSource._set_values({
    'VC_INHERITED': ProxySettingsSource('VC_INHERITED'),
    'CLUSTER_CONFIGURED': ProxySettingsSource('CLUSTER_CONFIGURED'),
    'NONE': ProxySettingsSource('NONE'),
})
ProxySettingsSource._set_binding_type(type.EnumType(
    'com.vmware.vcenter.namespace_management.supervisors.networks.management.proxy_settings_source',
    ProxySettingsSource))




class Network(VapiStruct):
    """
    
    
    ``Network`` class represents configuration for a network used to manage the
    Supervisor control plane. vCenter and, if used, NSX Manager and/or external
    Load Balancers, etc. should be reachable on this network.. This class was
    added in vSphere API 8.0.0.1.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """



    _canonical_to_pep_names = {
                            'floating_IP_address': 'floating_ip_address',
                            }

    def __init__(self,
                 network=None,
                 backing=None,
                 services=None,
                 ip_management=None,
                 floating_ip_address=None,
                 proxy=None,
                ):
        """
        :type  network: :class:`str` or ``None``
        :param network: 
            
            :attr:`Network.network` is a unique identifier for this network
            which can be referenced later for updates or queries.. This
            attribute was added in vSphere API 8.0.0.1.
            When clients pass a value of this class as a parameter, the
            attribute must be an identifier for the resource type:
            ``com.vmware.vcenter.namespace_management.Network``. When methods
            return a value of this class as a return value, the attribute will
            be an identifier for the resource type:
            ``com.vmware.vcenter.namespace_management.Network``.
            If unset, an ID will be generated.
        :type  backing: :class:`NetworkBacking`
        :param backing: 
            
            :attr:`Network.backing` specifies the network backing to use as the
            uplink to the management network.. This attribute was added in
            vSphere API 8.0.0.1.
        :type  services: :class:`com.vmware.vcenter.namespace_management.networks_client.Services` or ``None``
        :param services: 
            
            :attr:`Network.services` specifies which network services are
            configured on this network. These network services are expected to
            be accessible via the associated distributed virtual port group or
            distributed virtual switch.. This attribute was added in vSphere
            API 8.0.0.1.
            If unset, network services may be automatically configured. If you
            want to disable any configuration of network services, explicitly
            set this class to empty.
        :type  ip_management: :class:`com.vmware.vcenter.namespace_management.networks_client.IPManagement` or ``None``
        :param ip_management: 
            
            :attr:`Network.ip_management` describes how IP addressing is
            configured on this network.. This attribute was added in vSphere
            API 8.0.0.1.
            If unset, this network becomes a DHCP network. Your DHCP server
            must support client identifiers to successfully enable a
            Supervisor.
        :type  floating_ip_address: :class:`str` or ``None``
        :param floating_ip_address: 
            
            :attr:`Network.floating_ip_address` is an IP address that provides
            a stable endpoint to the control plane. This address if specified
            must be an unused statically allocated IP address on the management
            network.. This attribute was added in vSphere API 8.0.0.1.
            If None and the management network is a static network, this
            address should be allocated in a static IP configuration which
            occurs when
            :attr:`com.vmware.vcenter.namespace_management.networks_client.IPManagement.dhcp_enabled`
            is set to false. If None and the management network is a DHCP
            network, The Supervisor will attempt to use the DHCP server to
            allocate this address. You should ensure the DHCP server is
            configured to persist IP addresses indefinitely using client
            identifiers if this configuration is used.
        :type  proxy: :class:`ProxyConfiguration` or ``None``
        :param proxy: 
            
            Proxy configuration will be applied to the Supervisor. The proxy
            should be reachable from the management network and will be used
            for image pulling and container traffic exiting out of the
            Supervisor.. This attribute was added in vSphere API 8.0.0.1.
            If None the settings will be inherited from the vCenter settings if
            available.
        """
        self.network = network
        self.backing = backing
        self.services = services
        self.ip_management = ip_management
        self.floating_ip_address = floating_ip_address
        self.proxy = proxy
        VapiStruct.__init__(self)


Network._set_binding_type(type.StructType(
    'com.vmware.vcenter.namespace_management.supervisors.networks.management.network', {
        'network': type.OptionalType(type.IdType()),
        'backing': type.ReferenceType(__name__, 'NetworkBacking'),
        'services': type.OptionalType(type.ReferenceType('com.vmware.vcenter.namespace_management.networks_client', 'Services')),
        'ip_management': type.OptionalType(type.ReferenceType('com.vmware.vcenter.namespace_management.networks_client', 'IPManagement')),
        'floating_IP_address': type.OptionalType(type.StringType()),
        'proxy': type.OptionalType(type.ReferenceType(__name__, 'ProxyConfiguration')),
    },
    Network,
    False,
    None))



class NetworkBacking(VapiStruct):
    """
    
    
    A ``NetworkBacking`` enumerates the possible options for uplinking to a
    Supervisor management network.. This class was added in vSphere API
    8.0.0.1.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """

    _validator_list = [
        UnionValidator(
            'backing',
            {
                'NETWORK' : [('network', True)],
            }
        ),
    ]



    def __init__(self,
                 backing=None,
                 network=None,
                ):
        """
        :type  backing: :class:`NetworkBackingEnum`
        :param backing: 
            
            Selects the backing used for a management network.. This attribute
            was added in vSphere API 8.0.0.1.
        :type  network: :class:`str`
        :param network: 
            
            The Managed Object ID of the Network object.. This attribute was
            added in vSphere API 8.0.0.1.
            When clients pass a value of this class as a parameter, the
            attribute must be an identifier for the resource type: ``Network``.
            When methods return a value of this class as a return value, the
            attribute will be an identifier for the resource type: ``Network``.
            This attribute is optional and it is only relevant when the value
            of ``backing`` is :attr:`NetworkBackingEnum.NETWORK`.
        """
        self.backing = backing
        self.network = network
        VapiStruct.__init__(self)


NetworkBacking._set_binding_type(type.StructType(
    'com.vmware.vcenter.namespace_management.supervisors.networks.management.network_backing', {
        'backing': type.ReferenceType(__name__, 'NetworkBackingEnum'),
        'network': type.OptionalType(type.IdType()),
    },
    NetworkBacking,
    False,
    None))



class ProxyConfiguration(VapiStruct):
    """
    
    
    The ``ProxyConfiguration`` class defines proxy configuration to be used by
    the Supervisor.. This class was added in vSphere API 8.0.0.1.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """

    _validator_list = [
        UnionValidator(
            'proxy_settings_source',
            {
                'CLUSTER_CONFIGURED' : [('https_proxy_config', False), ('http_proxy_config', False), ('no_proxy_config', False), ('tls_root_ca_bundle', False)],
                'VC_INHERITED' : [],
                'NONE' : [],
            }
        ),
    ]



    def __init__(self,
                 proxy_settings_source=None,
                 https_proxy_config=None,
                 http_proxy_config=None,
                 no_proxy_config=None,
                 tls_root_ca_bundle=None,
                ):
        """
        :type  proxy_settings_source: :class:`ProxySettingsSource`
        :param proxy_settings_source: 
            
            The source of the proxy settings. 
            
            If :attr:`ProxySettingsSource.VC_INHERITED` or
            :attr:`ProxySettingsSource.NONE` is specified, then the other
            configuration in ``ProxyConfiguration`` will be ignored.. This
            attribute was added in vSphere API 8.0.0.1.
        :type  https_proxy_config: :class:`str` or ``None``
        :param https_proxy_config: 
            
            HTTPS proxy configuration. Examples: 
            
            * http://username:password\\\\@proxy.vmware.com:8080
            * https://proxy.vmware.com:4443
            
            
            
            This will be used if :attr:`ProxySettingsSource.CLUSTER_CONFIGURED`
            is used for the source, otherwise this will be ignored.. This
            attribute was added in vSphere API 8.0.0.1.
            If None no HTTPS proxy will be used.
        :type  http_proxy_config: :class:`str` or ``None``
        :param http_proxy_config: 
            
            HTTP proxy configuration. Examples: 
            
            * http://username:password\\\\@proxy.vmware.com:8080
            * https://proxy.vmware.com:4443
            
            
            
            This will be used if :attr:`ProxySettingsSource.CLUSTER_CONFIGURED`
            is used for the source, otherwise this will be ignored.. This
            attribute was added in vSphere API 8.0.0.1.
            If None no HTTP proxy will be used.
        :type  no_proxy_config: :class:`list` of :class:`str` or ``None``
        :param no_proxy_config: 
            
            List of addresses that should be accessed directly. 
            
            This will be used if :attr:`ProxySettingsSource.CLUSTER_CONFIGURED`
            is used for the source, otherwise this will be ignored.. This
            attribute was added in vSphere API 8.0.0.1.
            If None there won't be any excluded addresses.
        :type  tls_root_ca_bundle: :class:`str` or ``None``
        :param tls_root_ca_bundle: 
            
            Proxy TLS root CA bundle which will be used to verify the proxy's
            certificates. Every certificate in the bundle is expected to be in
            PEM format. 
            
            This will be used if :attr:`ProxySettingsSource.CLUSTER_CONFIGURED`
            is used for the source, otherwise this will be ignored.. This
            attribute was added in vSphere API 8.0.0.1.
            If None only the vCenter certificates applied in VECS (VMware
            Endpoint Certificate Store) will be used.
        """
        self.proxy_settings_source = proxy_settings_source
        self.https_proxy_config = https_proxy_config
        self.http_proxy_config = http_proxy_config
        self.no_proxy_config = no_proxy_config
        self.tls_root_ca_bundle = tls_root_ca_bundle
        VapiStruct.__init__(self)


ProxyConfiguration._set_binding_type(type.StructType(
    'com.vmware.vcenter.namespace_management.supervisors.networks.management.proxy_configuration', {
        'proxy_settings_source': type.ReferenceType(__name__, 'ProxySettingsSource'),
        'https_proxy_config': type.OptionalType(type.StringType()),
        'http_proxy_config': type.OptionalType(type.StringType()),
        'no_proxy_config': type.OptionalType(type.ListType(type.StringType())),
        'tls_root_ca_bundle': type.OptionalType(type.StringType()),
    },
    ProxyConfiguration,
    False,
    None))




class StubFactory(StubFactoryBase):
    _attrs = {
    }




# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.kms.providers.client_certificate.
#---------------------------------------------------------------------------

"""
The
``com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.kms.providers.client_certificate_client``
module provides the interfaces for configuring the Client Certificate used by
the Key Provider Service to authenticate against the Key Servers.

"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from com.vmware.cis_client import Tasks
from vmware.vapi.stdlib.client.task import Task
from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class Csr(VapiInterface):
    """
    The ``Csr`` interface provides methods to create a certificate signing
    request(CSR). This class was added in vSphere API 7.0.0.0.
    """

    _VAPI_SERVICE_ID = 'com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.kms.providers.client_certificate.csr'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _CsrStub)
        self._VAPI_OPERATION_IDS = {}
        self._VAPI_OPERATION_IDS.update({'create_task': 'create$task'})
        self._VAPI_OPERATION_IDS.update({'get_task': 'get$task'})

    class Info(VapiStruct):
        """
        The ``Csr.Info`` class contains the certificate signing request. This class
        was added in vSphere API 7.0.0.0.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     csr=None,
                    ):
            """
            :type  csr: :class:`str`
            :param csr: Certificate signing request. This attribute was added in vSphere
                API 7.0.0.0.
            """
            self.csr = csr
            VapiStruct.__init__(self)


    Info._set_binding_type(type.StructType(
        'com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.kms.providers.client_certificate.csr.info', {
            'csr': type.StringType(),
        },
        Info,
        False,
        None))




    def create_task(self,
               cluster,
               provider,
               ):
        """
        Generate a certificate signing request (CSR) for the client
        certificate. This overwrites any existing CSR. 
        
        The CSR will allow the certificate to be signed by a third party. Once
        the CSR is signed, certificate may be added as a client certificate. 
        
        Calling the API repeatedly will result in a generating a new CSR each
        time.. This method was added in vSphere API 7.0.0.0.

        :type  cluster: :class:`str`
        :param cluster: Identifier of the cluster.
            The parameter must be an identifier for the resource type:
            ``ClusterComputeResource``.
        :type  provider: :class:`str`
        :param provider: Identifier of the provider.
            The parameter must be an identifier for the resource type:
            ``com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.kms.Provider``.
        :rtype: :class:  `vmware.vapi.stdlib.client.task.Task`
        :return: Task instance
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidArgument` 
            If cluster or provider id are empty.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            If the provider or cluster is not found.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            If the caller is not authenticated.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If any other error occurs.
        """
        task_id = self._invoke('create$task',
                                {
                                'cluster': cluster,
                                'provider': provider,
                                })
        task_svc = Tasks(self._config)
        task_instance = Task(task_id, task_svc, type.ReferenceType(__name__, 'Csr.Info'))
        return task_instance


    def get_task(self,
            cluster,
            provider,
            ):
        """
        Get existing certificate signing request (CSR) for the client
        certificate. 
        
        The CSR will allow the certificate to be signed by a third party. Once
        the CSR is signed, certificate may be added as a client certificate. 
        
        . This method was added in vSphere API 7.0.0.0.

        :type  cluster: :class:`str`
        :param cluster: Identifier of the cluster.
            The parameter must be an identifier for the resource type:
            ``ClusterComputeResource``.
        :type  provider: :class:`str`
        :param provider: Identifier of the provider.
            The parameter must be an identifier for the resource type:
            ``com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.kms.Provider``.
        :rtype: :class:  `vmware.vapi.stdlib.client.task.Task`
        :return: Task instance
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidArgument` 
            If cluster or provider id are empty.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            If the provider or cluster is not found.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            If the caller is not authenticated.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If any other error occurs.
        """
        task_id = self._invoke('get$task',
                                {
                                'cluster': cluster,
                                'provider': provider,
                                })
        task_svc = Tasks(self._config)
        task_instance = Task(task_id, task_svc, type.ReferenceType(__name__, 'Csr.Info'))
        return task_instance
class _CsrStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for create operation
        create_input_type = type.StructType('operation-input', {
            'cluster': type.IdType(resource_types='ClusterComputeResource'),
            'provider': type.IdType(resource_types='com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.kms.Provider'),
        })
        create_error_dict = {
            'com.vmware.vapi.std.errors.invalid_argument':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidArgument'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        create_input_value_validator_list = [
        ]
        create_output_validator_list = [
        ]
        create_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/vcenter/trusted-infrastructure/trust-authority-clusters/{cluster}/kms/providers/{provider}/client-certificate/csr',
            path_variables={
                'cluster': 'cluster',
                'provider': 'provider',
            },
            query_parameters={
            },
            dispatch_parameters={
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'cluster': type.IdType(resource_types='ClusterComputeResource'),
            'provider': type.IdType(resource_types='com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.kms.Provider'),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_argument':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidArgument'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/vcenter/trusted-infrastructure/trust-authority-clusters/{cluster}/kms/providers/{provider}/client-certificate/csr',
            path_variables={
                'cluster': 'cluster',
                'provider': 'provider',
            },
            query_parameters={
            },
            dispatch_parameters={
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        operations = {
            'create$task': {
                'input_type': create_input_type,
                'output_type': type.IdType(resource_types='com.vmware.cis.TASK'),
                'errors': create_error_dict,
                'input_value_validator_list': create_input_value_validator_list,
                'output_validator_list': [],
                'task_type': TaskType.TASK_ONLY,
            },
            'get$task': {
                'input_type': get_input_type,
                'output_type': type.IdType(resource_types='com.vmware.cis.TASK'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': [],
                'task_type': TaskType.TASK_ONLY,
            },
        }
        rest_metadata = {
            'create': create_rest_metadata,
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.kms.providers.client_certificate.csr',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=True)


class StubFactory(StubFactoryBase):
    _attrs = {
        'Csr': Csr,
    }




"""
Task Helper for clients
"""
__author__ = 'VMware, Inc.'
__copyright__ = 'Copyright 2017 VMware, Inc.  All rights reserved. -- VMware Confidential'  # pylint: disable=line-too-long

from vmware.vapi.bindings.converter import TypeConverter


class Task(object):
    """
    Task Helper class for clients
    """
    def __init__(self, task_id, task_svc_instance, result_type):
        """
        Initialize the Task Helper class

        :type  task_id: :class:`str`
        :param task_id: Task Id
        :type  task_svc_instance: :class:`com.vmware.cis_client.Tasks`
        :param task_svc_instance: Task Service instance
        :type  result_type: :class:`vmware.vapi.bindings.BindingType`
        :param result_type: Result type
        """
        self.task_id = task_id
        self.task_svc_instance = task_svc_instance
        self.result_type = result_type

    def _get_task_svc_instance(self):
        """
        Returns tasks service instance

        :rtype: :class:`com.vmware.cis_client.Tasks`
        :return: Tasks service instance
        """
        return self.task_svc_instance

    def get_task_id(self):
        """
        Returns task id

        :rtype: :class:`str`
        :return: Task Id
        """
        return self.task_id

    def get_info(self):
        """
        Returns tasks info

        :rtype: :class:`com.vmware.cis.tasks_client.Info`
        :return: Task Info
        """
        return self.task_svc_instance.get(self.task_id)

    def get_result(self):
        """
        Returns task result

        :rtype: :class:`com.vmware.cis.tasks_client.Info`
        :return: Task Result
        """
        info = self.get_info()
        if hasattr(info, 'result'):
            return TypeConverter.convert_to_python(info.result,
                                                   self.result_type)
        else:
            return None

    def get_error(self):
        """
        Returns task error

        :rtype: :class:`Exception`
        :return: Task Error
        """
        return self.get_info().error



# -*- coding: utf-8 -*-

"""
SSO Security Helper
"""

__author__ = 'VMware, Inc.'
__copyright__ = 'Copyright 2015, 2017-2019, 2021 VMware, Inc.  All rights reserved. -- VMware Confidential'  # pylint: disable=line-too-long

from base64 import b64encode, b64decode
import datetime
import decimal
import json
from OpenSSL import crypto
import re
import six

from vmware.vapi.bindings.datetime_helper import DateTimeConverter
from vmware.vapi.core import SecurityContext
from vmware.vapi.lib.jsonlib import (
    DecimalEncoder, canonicalize_double)
from vmware.vapi.lib.constants import (
    PARAMS, SCHEME_ID, EXECUTION_CONTEXT, SECURITY_CONTEXT)
from vmware.vapi.protocol.common.lib import RequestProcessor

key_regex = re.compile(r'-----BEGIN [A-Z ]*PRIVATE KEY-----\n')
SAML_SCHEME_ID = 'com.vmware.vapi.std.security.saml_hok_token'
SAML_BEARER_SCHEME_ID = 'com.vmware.vapi.std.security.saml_bearer_token'
PRIVATE_KEY = 'privateKey'
SAML_TOKEN = 'samlToken'
SIGNATURE_ALGORITHM = 'signatureAlgorithm'
DEFAULT_ALGORITHM_TYPE = 'RS256'
TIMESTAMP = 'timestamp'
EXPIRES = 'expires'
CREATED = 'created'
REQUEST_VALIDITY = 20
SIGNATURE = 'signature'
DIGEST = 'value'
AUTHENTICATED = 'requestAuthenticated'
STS_URL_PROP = 'stsurl'
CERTIFICATE_PROP = 'certificate'
PRIVATE_KEY_PROP = 'privatekey'
CERT_NS = '{http://www.w3.org/2000/09/xmldsig#}'
SECTION = __name__
# Algorithm Header Parameter Values for JWS based on the following link
# https://datatracker.ietf.org/doc/draft-ietf-jose-json-web-algorithms/?include_text=1
algorithm_map = {
    'RS256': 'sha256',
    'RS384': 'sha384',
    'RS512': 'sha512',
}


def create_saml_bearer_security_context(token):
    """
    Create a security context for SAML bearer token based
    authentication scheme

    :type  token: :class:`str`
    :param token: SAML Token
    """
    return SecurityContext({SCHEME_ID: SAML_BEARER_SCHEME_ID,
                            SAML_TOKEN: token})


def create_saml_security_context(token, private_key):
    """
    Create a security context for SAML token based
    authentication scheme

    :type  token: :class:`str`
    :param token: SAML Token
    :type  private_key: :class:`str`
    :param private_key: Absolute file path of the private key of the user
    :rtype: :class:`vmware.vapi.core.SecurityContext`
    :return: Newly created security context
    """
    private_key_data = None
    with open(private_key, 'r') as fp:
        private_key_data = fp.read()
    return SecurityContext({SCHEME_ID: SAML_SCHEME_ID,
                            PRIVATE_KEY: private_key_data,
                            SAML_TOKEN: token,
                            SIGNATURE_ALGORITHM: DEFAULT_ALGORITHM_TYPE})


class JSONCanonicalEncoder(json.JSONEncoder):
    """
    Custom JSON Encoder class to canonicalize dictionary
    and list objects
    """
    def encode(self, o):
        """
        Encode a given python object

        :type  o: :class:`object`
        :param o: Python object
        :rtype: :class:`str`
        :return: JSON string in canonicalized form
        """
        if isinstance(o, dict):
            # Remove non-significant whitespace characters
            # Keys are sorted lexicographically using UCS code
            # point values
            sorted_keys = sorted(o.keys())
            sorted_items = ['%s:%s' % (self.encode(key),
                                       self.encode(o[key]))
                            for key in sorted_keys]
            string = ','.join(sorted_items)
            return '{%s}' % string
        elif isinstance(o, list):
            # Arrays must preserve the initial ordering
            # Remove non-significant whitespace characters
            string = ','.join([self.encode(item)
                               for item in o])
            return '[%s]' % string
        elif isinstance(o, decimal.Decimal):
            return canonicalize_double(o)
        else:
            return json.JSONEncoder.encode(self, o)


class JSONCanonicalizer(object):
    """
    This class is responsible for transforming JSON messages into their
    canonical representation.

    The canonical form is defined by the following rules:
        1. Non-significant(1) whitespace characters MUST NOT be used
        2. Non-significant(1) line endings MUST NOT be used
        3. Entries (set of name/value pairs) in JSON objects MUST be sorted
           lexicographically(2) by their names based on UCS codepoint values
        4. Arrays MUST preserve their initial ordering

    Link to the IEFT proposal:
    https://datatracker.ietf.org/doc/draft-staykov-hu-json-canonical-form/
    """
    @staticmethod
    def canonicalize(input_message):
        """
        Canonicalize the input message

        :type  input_message: :class:`str`
        :param input_message: Input message
        :rtype: :class:`str`
        :return: Canonicalized message
        """
        py_obj = json.loads(input_message, parse_float=decimal.Decimal)
        return JSONCanonicalEncoder().encode(py_obj)

    @staticmethod
    def canonicalize_py_obj(py_obj):
        """
        Canonicalize the input python object

        :type  input_message: :class:`object`
        :param input_message: Input python object
        :rtype: :class:`str`
        :return: Canonicalized message
        """
        return JSONCanonicalEncoder().encode(py_obj)


class JSONSSOSigner(RequestProcessor):
    """
    This class is used for signing JSON request messages
    """
    def process(self, input_message):
        """
        Sign the input JSON request message.

        The message is signed using user's private key. The digest and saml
        token is then added to the security context block of the execution
        context. A timestamp is also added to guard against replay attacks

        Sample input security context:
        {
            'schemeId': 'SAML_TOKEN',
            'privateKey': <PRIVATE_KEY>,
            'samlToken': <SAML_TOKEN>,
            'signatureAlgorithm': <ALGORITHM>,
        }

        Security context block before signing:
        {
            'schemeId': 'SAML_TOKEN',
            'signatureAlgorithm': <ALGORITHM>,
            'timestamp': {
                'created': '2012-10-26T12:24:18.941Z',
                'expires': '2012-10-26T12:44:18.941Z',
            }
        }

        Security context block after signing:
        {
            'schemeId': 'SAML_TOKEN',
            'signatureAlgorithm': <ALGORITHM>,
            'signature': {
                'samlToken': <SAML_TOKEN>,
                'value': <DIGEST>
            }
            'timestamp': {
                'created': '2012-10-26T12:24:18.941Z',
                'expires': '2012-10-26T12:44:18.941Z',
            }
        }
        """
        if input_message is None:
            return None

        # process only if the schemeId in the request matches the schemeId of
        # this signer
        if isinstance(input_message, six.binary_type):
            unicode_input_message = input_message.decode()
        else:
            unicode_input_message = input_message
        if SAML_SCHEME_ID not in unicode_input_message:
            return input_message

        py_obj = json.loads(unicode_input_message, parse_float=decimal.Decimal)
        json_params = py_obj.get(PARAMS)  # pylint: disable=E1103
        ctx = json_params.get(EXECUTION_CONTEXT)

        sec_ctx = ctx.get(SECURITY_CONTEXT)
        private_key = sec_ctx.get(PRIVATE_KEY)
        saml_token = sec_ctx.get(SAML_TOKEN)
        jws_algorithm = sec_ctx.get(SIGNATURE_ALGORITHM)
        algorithm = algorithm_map.get(jws_algorithm)

        new_sec_ctx = {}
        new_sec_ctx[SCHEME_ID] = sec_ctx.get(SCHEME_ID)
        new_sec_ctx[TIMESTAMP] = _generate_request_timestamp()
        new_sec_ctx[SIGNATURE_ALGORITHM] = jws_algorithm

        # Replace the old security context with the new one
        del ctx[SECURITY_CONTEXT]
        ctx[SECURITY_CONTEXT] = new_sec_ctx

        pkey = crypto.load_privatekey(
            crypto.FILETYPE_PEM, _prep_private_key(private_key))
        canonical_message = JSONCanonicalizer.canonicalize_py_obj(py_obj)
        base64_encoded_digest = b64encode(crypto.sign(
            pkey, canonical_message.encode(), algorithm))
        # Convert to unicode since json library does not accept bytes
        unicode_digest = base64_encoded_digest.decode()

        new_sec_ctx[SIGNATURE] = {SAML_TOKEN: saml_token,
                                  DIGEST: unicode_digest}
        return json.dumps(py_obj,
                          check_circular=False,
                          separators=(',', ':'),
                          cls=DecimalEncoder)


class JSONSSOVerifier(RequestProcessor):
    """
    This class is used to verify the authenticity of the request
    message by verifying the digest present in the security context
    block.
    """
    def process(self, input_message):
        """
        Verify the input JSON message.

        For verification, we need 4 things:

        1. algorithm: extracted from security context
        2. certificate: public key of the principal embedded in the
        SAML token is used
        3. digest: value field from signature block
        4. canonical msg: signature block is removed from the request
        and the remaining part is canonicalized

        Sample input security context:
        {
            'schemeId': 'SAML_TOKEN',
            'signatureAlgorithm': <ALGORITHM>,
            'signature': {
                'samlToken': <SAML_TOKEN>,
                'value': <DIGEST>
            }
            'timestamp': {
                'created': '2012-10-26T12:24:18.941Z',
                'expires': '2012-10-26T12:44:18.941Z',
            }
        }

        :type  input_message: :class:`str`
        :param input_message: Input JSON request message
        :rtype: :class:`str`
        :return: JSON request message after signature verification
        """
        if not input_message:
            return None

        if isinstance(input_message, six.binary_type):
            unicode_input_message = input_message.decode()
        else:
            unicode_input_message = input_message
        if SAML_SCHEME_ID not in unicode_input_message:
            return input_message

        py_obj = json.loads(unicode_input_message, parse_float=decimal.Decimal)
        json_params = py_obj.get(PARAMS)  # pylint: disable=E1103
        execution_ctx = json_params.get(EXECUTION_CONTEXT)
        sec_ctx = execution_ctx.get(SECURITY_CONTEXT)

        signature = sec_ctx.get(SIGNATURE)
        del sec_ctx[SIGNATURE]

        base64_encoded_digest = signature.get(DIGEST).encode()
        digest = b64decode(base64_encoded_digest)
        jws_algorithm = sec_ctx.get(SIGNATURE_ALGORITHM)
        algorithm = algorithm_map.get(jws_algorithm)
        saml_token = signature.get(SAML_TOKEN)
        certificate = _extract_certificate(saml_token)

        pubkey = crypto.load_certificate(
            crypto.FILETYPE_PEM, _prep_certificate(certificate))
        canonical_message = JSONCanonicalizer.canonicalize_py_obj(py_obj)
        crypto.verify(
            pubkey, digest, canonical_message.encode(), algorithm)

        sec_ctx[SAML_TOKEN] = saml_token
        sec_ctx[AUTHENTICATED] = True
        sec_ctx[SIGNATURE_ALGORITHM] = jws_algorithm

        return json.dumps(py_obj,
                          check_circular=False,
                          separators=(',', ':'),
                          cls=DecimalEncoder)


def _extract_element(xml, element_name, namespace):
    """
    An internal method provided to extract an element from the given XML.

    :type  xml: :class:`str`
    :param xml: The XML string from which the element will be extracted.
    :type  element_name: :class:`str`
    :param element_name: The element that needs to be extracted from the XML.
    :type  namespace: :class:`dict`
    :param namespace: A dict containing the namespace of the element to be
        extracted.
    :rtype: etree element.
    :return: The extracted element.
    """
    assert(len(namespace) == 1)
    result = xml.xpath("//%s:%s" % (list(namespace.keys())[0], element_name),
                       namespaces=namespace)
    if result and len(result) == 1:
        return result[0]
    else:
        raise KeyError("%s does not seem to be present or valid in the XML." %
                       element_name)


def _prep_private_key(private_key):
    """
    Append proper prefix and suffix text to a private key. There is no
    standard way for storing certificates. OpenSSL expects the demarcation
    text. This method makes sure that the text the markers are present.

    :type text: :class:`str`
    :param text: The private key of the service user.

    :rtype: :class:`str`
    :return: Normalized private key.
    """
    if not key_regex.search(private_key):
        return """-----BEGIN RSA PRIVATE KEY-----
%s
-----END RSA PRIVATE KEY-----""" % private_key
    return private_key


def _prep_certificate(certificate):
    """
    Append proper prefix and suffix text to a certificate. There is no
    standard way for storing certificates. OpenSSL expects the demarcation
    text. This method makes sure that the text the markers are present.

    :type text: :class:`str`
    :param text: The certificate of the service user.

    :rtype: :class:`str`
    :return: Normalized certificate
    """
    if not certificate.startswith('-----BEGIN CERTIFICATE-----'):
        return """-----BEGIN CERTIFICATE-----
%s
-----END CERTIFICATE-----""" % certificate
    return certificate


def _extract_certificate(hok_token):
    """
    Extract Certificate of the principal from Holder of Key SAML token

    :type  hok_token: :class:`str`
    :param hok_token: Holder of key SAML token
    :rtype: :class:`str`
    :return: Certificate of the principal
    """
    xml = parse_xml(hok_token)
    subject = _extract_element(
        xml,
        'SubjectConfirmationData',
        {'saml2': 'urn:oasis:names:tc:SAML:2.0:assertion'})

    xml_certificate = subject.find('./' + CERT_NS + 'KeyInfo/'
                                   + CERT_NS + 'X509Data/'
                                   + CERT_NS + 'X509Certificate')
    xml_certificate = _get_element_text(xml_certificate)
    return xml_certificate.replace('\\n', '\n')


def parse_xml(xml):
    from lxml import etree
    p = etree.XMLParser(resolve_entities=False)
    return etree.fromstring(xml, p)


def _generate_request_timestamp():
    """
    Generate a timestamp for the request. This will be embedded in the security
    context of the request to protect it against replay attacks

    :rtype: :class:`dict`
    :return: Timestamp block that can be inserted in security context
    """
    created_dt = datetime.datetime.utcnow()
    offset = datetime.timedelta(minutes=REQUEST_VALIDITY)
    created = DateTimeConverter.convert_from_datetime(created_dt)
    expires = DateTimeConverter.convert_from_datetime(created_dt + offset)
    return {EXPIRES: expires, CREATED: created}


def _get_element_text(element):
    """
    Reading element.text is unsafe because of comments that may be embedded
    in certain important fields by an attacker.
    To ensure that we're getting the whole contents of the field this
    method concatenates all text children of the given element.

    Sample:
    <name>root<!-- -->less-user</name> --> "rootless-user"
    <name>alpha<x>bravo</x>charlie</name> --> "alpha"
    <name></name> --> None

    :type  element: :class:`str`
    :param element: XML document element
    :rtype: :class:`str`
    :return: XML document stripped off comment
    """
    from lxml import etree
    parts = []
    if element.text is not None:
        parts.append(element.text)
    for child in element.iterchildren():
        if child.tag is etree.Comment:
            if child.tail is not None:
                parts.append(child.tail)
        else:
            break
    if not parts:
        return None
    return "".join(parts)



"""
Task Manager Interface
"""

__author__ = 'VMware, Inc.'
__copyright__ = 'Copyright 2017-2018 VMware, Inc.  All rights reserved. -- VMware Confidential'  # pylint: disable=line-too-long

import abc
import six


@six.add_metaclass(abc.ABCMeta)
class TaskManager(object):
    """
    The TaskManager interface providing methods to manage tasks
    """

    @abc.abstractmethod
    def create_task(self, description, service_id, operation_id,
                    cancelable, error_types, id_):
        """
        Creates a task in task manager.

        :type  description: :class:`com.vmware.vapi.std.LocalizableMessage`
        :param description: Task description.
        :type  service_id: :class:`str`
        :param service_id: Service Id.
        :type  operation_id: :class:`str`
        :param operation_id: Operation Id.
        :type  cancelable: :class:`bool`
        :param cancelable: Is the task cancelable.
        :type  error_types: :class:`list` of
            :class:`vmware.vapi.data.definition.ErrorDefinition`
        :param error_types: Error definitions describing the errors this
            operation can report
        :type  id_: :class:`str`
        :param id_: Base task id
        """
        pass

    @abc.abstractmethod
    def get_info(self, task_id):
        """
        Get task info for a given task.

        :type  task_id: :class:`str`
        :param task_id: Task Id.

        :rtype:  :class:`vmware.vapi.data.value.StructValue`
        :return: Task Info
        """
        pass

    @abc.abstractmethod
    def set_info(self, task_id, info):
        """
        Set task info.

        :type  task_id: :class:`str`
        :param task_id: Task Id.
        :type  status: :class:`vmware.vapi.data.value.StructValue`
        :param status: Task Info.
        """
        pass

    @abc.abstractmethod
    def remove_task(self, task_id):
        """
        Remove task from tasks map.

        :type  task_id: :class:`str`
        :param task_id: Task Id.
        """
        pass

    @abc.abstractmethod
    def get_infos(self):
        """
        Get the information of all tasks.

        :rtype:  :class:`dict` of :class:`str` and
                 :class:`vmware.vapi.data.value.StructValue`
        :return: Map of task identifier to information about the task.
        """
        pass

    @abc.abstractmethod
    def mark_for_cancellation(self, task_id):
        """
        Add a task to list of tasks to be canceled.
        It depends on provider implementation to cancel it or not.

        :type  task_id: :class:`str`
        :param task_id: Task Id.
        """
        pass

    @abc.abstractmethod
    def is_marked_for_cancellation(self, task_id):
        """
        Check whether a task is marked for cancellation.

        :type  task_id: :class:`str`
        :param task_id: Task Id.
        """
        pass
